/*
ͻ
   Libreria de fuentes multicolor ->  MULTIFNT.H                            
͹
  Librera grfica de tratamiento de fuentes multicolor.                    
  Se ha optado por incluir las funciones con mucho cdigo en ensamblador    
  ya que de lo contrario nos limita la velocidad en la ejecucin de nuestros
  programas.                                                                
                                                                            
  Para incluir la librera en un programa basta con aadir al listado       
  principal la orden: #include "multifnt.h" al principio del mismo.         
                                                                            
                                                                            
 Cdigo y texto: (:-> Madd <-:) de Compiler SoftWare                        
ͼ
*/

int ascii_inicial=0;

// DEFINICION DE FUNCIONES 


void SetDimension ( int , int );
    /*
    Funcin que debe ser llamada antes de utilizar cualquier fuentes de letras
    para poder definir previamente las dimensiones horizontales y verticales
    de la fuente que se desea utilizar. Una vez llamada ya no ser necesario
    volver a llamarla a no ser de tener que cambiar a otro estilo de fuente
    diferente.
    */

void WriteCharM1 ( int , int , char , char , char *);
    /*
    Mtodo n 1 de escritura de caracteres:
    Como en C ya sabemos anidar bucles para volcar sprites, ahora introducimos
    un poco ms de ensamblador.
    Escribe un caracter en el segmento de memoria deseado con las siguientes
    entradas: coordenada x, coordenada y, n de caracter, color de mscara ,
    array de la fuente.
    
    */
void WriteCharM2 ( int , int  , char , char , char *, int);
    /*
    Mtodo n 2 de escritura de caracteres:
    Este mtodo desecha totalmente cualquier tipo de estructura en C y limita
    su funcionamiento a registro y variables, con lo que le hace ganar mucha
    velocidad.
    Escribe un caracter en el segmento de memoria deseado con las siguientes
    entradas: coordenada x, coordenada y, n de caracter, el color de mscara,
    array de la fuente y segmento de memoria ( 0xA000 es la Video Ram )
    */
void WriteCharM3 ( int,int ,char, char, char far *, int );
    /*
     Mtodo n 3 con una rutina entera en ASM, que acelera los clculos
     en gran medida, tal y como mustra el test de ejemplo.
     */

void WriteStr ( int , int , char * , char , char *, int );
    /*
    Escribe una frase en el segmento de memoria deseado, y cuya funcin
    tiene las siguientes entradas: coord. x, coord. y, string de texto,
    color de mscara, array de la fuente y el segmento de memoria.
    */

// DEFINICION DE VARIBLES 


int MLANCHO , MLALTO;    // Contiene ancho y alto de la fuente

//

void SetDimension ( int ancho , int alto )
{
MLANCHO = ancho;
MLALTO = alto;
}



void WriteCharM1 ( int x, int y, char ncar, char masc , char *fuente)
{
int offset_car;
char bucle1 , bucle2;    // variables de bucles
unsigned char color;

offset_car = ncar*(MLANCHO*MLALTO);


for (bucle1 = 0;bucle1 < MLALTO;bucle1 ++)
    for (bucle2 = 0;bucle2 < MLANCHO;bucle2 ++)
        if (color != masc) PutPixel ( x+bucle2 , y+bucle1 , fuente[offset_car++]);
}

void WriteCharM2 ( int x, int y, char ncar, char masc , char *fuente, int segmento)
{
int offset_car;      // Offset de la fuente donde comienza el caracter
int offset_mem;      // Offset de la memoria donde pongo el caracter
char bucle1 , bucle2;// variables de bucles
char color;          // Color de pixel actual

offset_car = ncar * (MLANCHO*MLALTO);
         asm {
                mov ax , [segmento]
                mov es , ax
                mov ax , [y]
                mov bx , ax
                shl ax , 8
                shl bx , 6
                add ax , bx
                add ax , [x]
                mov di , ax
            }
  for (bucle1 = 0;bucle1 < MLALTO;bucle1 ++,y++)
  {
      for (bucle2 = 0;bucle2 < MLANCHO;bucle2 ++,offset_car++)
      {
                color = fuente[offset_car];
                asm {
                     mov al , [color]
                     cmp al , [masc]        /* color de la mscara: nico */
                     je NoPinta             /* color que no dibujar      */
                     mov es:[di] , al
                    }
                NoPinta:
                     asm inc di             /* incremento offset de pantalla */

      }
                asm {
                     mov bx , [MLANCHO]
                     sub di , bx
                     add di , 320
                    }

  }
}


void WriteCharM3 ( int x, int y, char ncar, char masc , char far *fuente, int segmento)
{
int offset_car;

offset_car = ncar * (MLANCHO*MLALTO);
asm {
    lds si , [fuente]            /* Cargo direccin de la fuente en DS:SI */
    add si , [offset_car]

    mov ax , [segmento]
    mov es , ax
    mov ax , [y]
    mov bx , ax
    shl ax , 8
    shl bx , 6
    add ax , bx
    add ax , [x]
    mov di , ax

    mov bx , [MLALTO]
    mov cx , [MLANCHO]
  }
bucle:
asm {
    mov al , ds:[si]
    cmp al , [masc]
    je NoPinta
    mov es:[di] , al
   }
NoPinta:
asm {
    inc di
    inc si
    dec cx
    jnz bucle

    mov cx , [MLANCHO]
    sub di , cx
    add di , 320
    dec bx
    jnz bucle
   }

}

void WriteStr ( int x, int y, char *texto , char masc ,char *fuente, int segmento)
{
int contador = 0;


 while (texto[contador] != 0)
 {
   WriteCharM3 (x , y , texto[contador]-ascii_inicial , 
                masc, fuente , segmento);
   contador++;
   x += (MLANCHO+5);  // 5 indica separacin entre caracteres
 }

}
